body { background-color: white; margin: 0px; padding: 0px; font-size: 12px; font-family: verdana; }
div.catName { padding: 10px; background-color: #70803C; font-size: 20px; color: white; }
div.catDesc { padding: 20px 20px 10px; }
ul.catService { }
ul.catService li { padding-bottom: 10px; }
div.catInterfaces { padding: 0px 10px 10px 30px; }
div.catGen { text-align: center; font-style: italic; margin-top: 20px; margin-bottom: 20px; } 
