# -*- coding: utf-8 -*-

catalog_default_css = '''
	body { background-color: white; margin: 0px; padding: 0px; font-size: 12px; font-family: "Lucida Console", Monaco, monospace; }
	div.catName { height: 22px; padding: 10px; background-color: #000; font-size: 20px; color: white; }
	div.catDesc { padding: 20px 20px 10px; }
	ul.catService { }
	ul.catService li { padding-bottom: 10px; }
	div.catInterfaces { padding: 0px 10px 10px 30px; }
	div.catGen { text-align: center; font-style: italic; margin-top: 20px; margin-bottom: 20px; } 
'''

service_default_css = '''
	body { background-color: white; margin: 0px; padding: 0px; font-size: 12px; font-family: "Lucida Console", Monaco, monospace; }
	div.serviceName { height: 22px; padding: 10px; background-color: #000; font-size: 20px; color: white; }
	div.title {position: relative; float:left}
	form.skin-selector {position:relative; float: right; font-size: 12pt;}
	div.serviceDesc { padding: 20px 20px 10px; }
	span.descriptionHead { font-weight: bold; }
	span.interfaceHead { padding-left: 20px; font-weight: bold; }
	span.methodsHead { padding-left: 20px; font-weight: bold; }
	span.typesHead { padding-left: 20px; font-weight: bold; }
	div.methodDef { font-size: 16px; font-family: courier }
	div.typeDef { font-size: 16px; font-family: courier }
	span.methodName { color: darkred; font-weight: bold; font-family: courier }
	span.typeName { color: darkred; font-weight: bold; font-family: courier }
	span.paramType { color: blue }
	span.paramName { color: black; font-weight: bold }
	li.paramDesc { padding-bottom: 10px }
	li.methodEntry { padding-bottom: 15px }
	li.typeAttrib { padding-bottom: 10px }
	p.docLines { margin-top: 4px; color: black; margin-bottom: 4px }
	ul.interfaceList { }
	ul.interfaceList li { padding-bottom: 10px; }
	div.catGen { text-align: center; font-style: italic; margin-top: 20px; }
'''
